@@ -1,5 +1,5 @@
 // =================================================================================================
-// Copyright 2002-2007 Adobe Systems Incorporated
+// Copyright 2002-2008 Adobe Systems Incorporated
 // All Rights Reserved.
 //
 // NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the terms
@@ -8,6 +8,7 @@
 
 #include "XMP_Environment.h"	// ! This must be the first include!
 #include "XMPCore_Impl.hpp"
+#include "ExpatAdapter.hpp"
 
 #include <cstring>
 
@@ -18,8 +19,10 @@
 using namespace std;
 
 #if XMP_WinBuild
-	#pragma warning ( disable : 4189 )	// local variable is initialized but not referenced
-	#pragma warning ( disable : 4505 )	// unreferenced local function has been removed
+#   ifdef _MSC_VER
+        #pragma warning ( disable : 4189 )	// local variable is initialized but not referenced
+        #pragma warning ( disable : 4505 )	// unreferenced local function has been removed
+#   endif
 #endif
 
 // =================================================================================================
@@ -463,8 +466,8 @@ AddQualifierNode ( XMP_Node * xmpParent, const XMP_VarString & name, const XMP_V
 			}
 			xmpParent->options |= kXMP_PropHasType;
 		}
-		
-		xmpParent->options |= kXMP_PropHasQualifiers;	// ! Don't set for the pxmp:compact.
+
+		xmpParent->options |= kXMP_PropHasQualifiers;
 
 	return newQual;
 
@@ -587,7 +590,7 @@ FixupQualifiedNode ( XMP_Node * xmpParent )
 	// value node's children to be the parent's children. Delete the now useless value node.
 	
 	XMP_Assert ( xmpParent->options & (kXMP_PropValueIsStruct | kRDF_HasValueElem) );
-	xmpParent->options &= ~ (kXMP_PropValueIsStruct | kRDF_HasValueElem);
+	xmpParent->options &= ~ (static_cast<unsigned long>(kXMP_PropValueIsStruct) | static_cast<unsigned long>(kRDF_HasValueElem));
 	xmpParent->options |= valueNode->options;
 	
 	xmpParent->value.swap ( valueNode->value );
@@ -664,7 +667,7 @@ RDF_NodeElementList ( XMP_Node * xmpParent, const XML_Node & xmlParent, bool isT
 	XML_cNodePos endChild  = xmlParent.content.end();
 
 	for ( ; currChild != endChild; ++currChild ) {
-		if ( IsWhitespaceNode ( **currChild ) ) continue;
+		if ( (*currChild)->IsWhitespaceNode() ) continue;
 		RDF_NodeElement ( xmpParent, **currChild, isTopLevel );
 	}
 
@@ -748,13 +751,10 @@ RDF_NodeElementAttrs ( XMP_Node * xmpParent, const XML_Node & xmlNode, bool isTo
 					// This is the rdf:about attribute on a top level node. Set the XMP tree name if
 					// it doesn't have a name yet. Make sure this name matches the XMP tree name.
 					XMP_Assert ( xmpParent->parent == 0 );	// Must be the tree root node.
-					if ( ! xmpParent->name.empty() ) {
-						if ( xmpParent->name != (*currAttr)->value ) XMP_Throw ( "Mismatched top level rdf:about values", kXMPErr_BadXMP );
-					} else {
+					if ( xmpParent->name.empty() ) {
 						xmpParent->name = (*currAttr)->value;
-						#if 0	// *** XMP_DebugBuild
-							xmpParent->_namePtr = xmpParent->name.c_str();
-						#endif
+					} else if ( ! (*currAttr)->value.empty() ) {
+						if ( xmpParent->name != (*currAttr)->value ) XMP_Throw ( "Mismatched top level rdf:about values", kXMPErr_BadXMP );
 					}
 				}
 
@@ -788,7 +788,7 @@ RDF_PropertyElementList ( XMP_Node * xmpParent, const XML_Node & xmlParent, bool
 	XML_cNodePos endChild  = xmlParent.content.end();
 
 	for ( ; currChild != endChild; ++currChild ) {
-		if ( IsWhitespaceNode ( **currChild ) ) continue;
+		if ( (*currChild)->IsWhitespaceNode() ) continue;
 		if ( (*currChild)->kind != kElemNode ) {
 			XMP_Throw ( "Expected property element node not found", kXMPErr_BadRDF );
 		}
@@ -846,7 +846,7 @@ RDF_PropertyElementList ( XMP_Node * xmpParent, const XML_Node & xmlParent, bool
 // are distinguished by their XML element content.
 //
 // NOTE: The RDF syntax does not explicitly include the xml:lang attribute although it can appear in
-// many of these. We have to allow for it in the attibute counts below.
+// many of these. We have to allow for it in the attribute counts below.
 
 static void
 RDF_PropertyElement ( XMP_Node * xmpParent, const XML_Node & xmlNode, bool isTopLevel )
@@ -962,7 +962,7 @@ RDF_ResourcePropertyElement ( XMP_Node * xmpParent, const XML_Node & xmlNode, bo
 	XML_cNodePos endChild  = xmlNode.content.end();
 
 	for ( ; currChild != endChild; ++currChild ) {
-		if ( ! IsWhitespaceNode ( **currChild ) ) break;
+		if ( ! (*currChild)->IsWhitespaceNode() ) break;
 	}
 	if ( currChild == endChild ) XMP_Throw ( "Missing child of resource property element", kXMPErr_BadRDF );
 	if ( (*currChild)->kind != kElemNode ) XMP_Throw ( "Children of resource property element must be XML elements", kXMPErr_BadRDF );
@@ -992,7 +992,7 @@ RDF_ResourcePropertyElement ( XMP_Node * xmpParent, const XML_Node & xmlNode, bo
 	}
 
 	for ( ++currChild; currChild != endChild; ++currChild ) {
-		if ( ! IsWhitespaceNode ( **currChild ) ) XMP_Throw ( "Invalid child of resource property element", kXMPErr_BadRDF );
+		if ( ! (*currChild)->IsWhitespaceNode() ) XMP_Throw ( "Invalid child of resource property element", kXMPErr_BadRDF );
 	}
 
 }	// RDF_ResourcePropertyElement
